#!/usr/bin/env ruby
#
# RTDS - Ruby Tiny Diary System
# (C)2005 H.Miyamoto (aka "Yuukis")
#                       <Ys@PixyGarden.net>
#
# URI: http://wids.net/lab/rtds.html
#
#
# $Id: rtds.rb,v 1.52 2005/12/10 23:50:14 Yuukis Exp $
#
$KCODE = 'e'

class RTDS
	require 'cgi'
	require 'cgi/session'
	require 'erb'

	require 'uconv'
	require 'bluecloth'

	VERSION	= '0.7'
	CODE	= 'China Blue'
	ACRONYM	= '<abbr title="Ruby Tiny Diary System">RTDS</abbr>'
	POWERED	= %Q(<a href="http://wids.net/lab/rtds.html">#{ACRONYM}</a> version #{VERSION} (#{CODE}))
	CFG		= 'rtds.conf'

	H2CLASS = 'date'  # tdiary theme compat.
	H3CLASS = 'article_title'
	DayClass = 'day'  # tdiary theme compat.
	SectionClass = 'section'  # tdiary theme compat.
	BodyClass = 'body'  # tdiary theme compat.
	ArticleBodyClass = 'article_body'
	SectionAnchor = '_'
	SectionAnchorClass = 'sanchor'

	def initialize
		cgi = CGI.new

		if cgi.params.has_key?('convert')
			begin
				@session = CGI::Session.new(cgi, {'new_session' => false})
			rescue
				print "Location: #{ENV['SCRIPT_NAME']}?mode=auth\n\n"
			end
			if File.exist?(CFG)
				load CFG
			else
				cgi.out('text/plain') { "#{CFG} doesn't exist." }
				exit
			end
			all_rebuild
			print "Location: ./\n\n"
			exit
		end

		mode = cgi['mode']
		if File.exist?(CFG)
			load CFG
		else
			cgi.out('text/plain') { "#{CFG} doesn't exist." }
			exit
		end
		if File.exist?(LOG_DIRECTORY) == false
			Dir.mkdir(LOG_DIRECTORY)
		end

		case mode
		when 'auth'
			erb = ERB.new(IO.readlines(AUTH_FILE).join)
			cgi.out('text/html') { erb.result(binding) }
			exit
		when 'edit'
			pass = IO.readlines(PASSWD_FILE).join
			if cgi['pass'].crypt(pass[0, 2]) == pass.chomp
			else
				cgi.out('text/plain') { "You could not identify yourself, please retry." }
				exit
			end

			@session = CGI::Session.new(cgi, {'new_session' => true})
			erb = ERB.new(IO.readlines(INPUT_FILE).join)
			cgi.out('text/html') { erb.result(binding) }
		when 'redirect'
			uri = cgi['uri']
			print "Location: ./#{uri}\n\n"
		when 'update'
			begin
				@session = CGI::Session.new(cgi, {'new_session' => false})
			rescue
				print "Location: #{ENV['SCRIPT_NAME']}?mode=auth\n\n"
			end

			begin
				date = Time.local(cgi['year'], cgi['month'], cgi['day'])
				@year = date.strftime("%Y")
				@month = date.strftime("%m")
				@day = date.strftime("%d")
			rescue
				cgi.out('text/plain') { "You inputed invalid date, please retry." }
				exit
			end

			DataStore.drop2log(@year, @month, @day, cgi['comment'])

			rebuild

			print "Location: ./\n\n"
		else
			print "Location: #{ENV['SCRIPT_NAME']}?mode=auth\n\n"
		end
	end

	def all_rebuild
		cgi = CGI.new
		files = []
		Dir.glob("#{LOG_DIRECTORY}*#{LOG_SUFFIX}").each {|fp|
			files << converted_file_name(fp.sub(/#{LOG_DIRECTORY}(\d{10})#{LOG_SUFFIX}/, '\1'))
		}
		files.uniq!
		files.sort!

		files.each {|fn|
			y = ''; m = ''; d = ''; suffix = ''
			fn.sub(/(\d{4})(\d{2})(.)#{SUFFIX}/) { y = $1; m = $2; suffix = $3 }
			if suffix == 'a'
				d = '01'
			else
				d = '16'
			end

			table = DiaryTable.new
			plugin = Plugin.new(y, m, d, suffix, table)

			diary = Diary.new(DIARY_BASE_FILE, y, m, d)
			diary_string = plugin.filtering(diary.to_html)
			DataStore.drop2file(diary_string, BASE_DIRECTORY + y + m + suffix + SUFFIX)
		}

		y = ''; m = ''; d = ''; suffix = ''
		last_log = files[files.size - 1]
		last_log.sub(/(\d{4})(\d{2})(.)#{SUFFIX}/) { y = $1; m = $2; suffix = $3 }
		if suffix == 'a'
			d = '01'
		else
			d = '16'
		end

		table = DiaryTable.new
		plugin = Plugin.new(y, m, d, suffix, table)

		diaryindex = DiaryIndex.new(DIARYINDEX_BASE_FILE)
		diaryindex_string = plugin.filtering(diaryindex.to_html)
		DataStore.drop2file(diaryindex_string, DIARYINDEX_FILE)

		idx = Index.new(INDEX_BASE_FILE)
		idx_string = plugin.filtering(idx.to_html)
		DataStore.drop2file(idx_string, INDEX_FILE)

		plugin.exec
	end

	def converted_file_name(fn)
		y = ''; m = ''; d = ''
		fn.sub(/(\d{4})(\d{2})(\d{2})\d{2}/) { y = $1; m = $2; d = $3 }
		if d.to_i < 16
			suffix = 'a'
		else
			suffix = 'b'
		end
		y + m + suffix + SUFFIX
	end

	def rebuild
		if @day.to_i < 16
			@day_suffix = 'a'
		else
			@day_suffix = 'b'
		end

		need_diary_index_refine = false # ugly...
		if File.exist?(BASE_DIRECTORY + @year + @month + @day_suffix + SUFFIX) == false
			need_diary_index_refine = true
			File.open(BASE_DIRECTORY + @year + @month + @day_suffix + SUFFIX, 'w') {|fh| fh.write ' ' }
		end

		@table = DiaryTable.new
		@plugin = Plugin.new(@year, @month, @day, @day_suffix, @table)

		if need_diary_index_refine
			diaryindex = DiaryIndex.new(DIARYINDEX_BASE_FILE)
			diaryindex_string = @plugin.filtering(diaryindex.to_html)
			DataStore.drop2file(diaryindex_string, DIARYINDEX_FILE)
		end

		diary = Diary.new(DIARY_BASE_FILE, @year, @month, @day)
		diary_string = @plugin.filtering(diary.to_html)
		DataStore.drop2file(diary_string, BASE_DIRECTORY + @year + @month + @day_suffix + SUFFIX)

		idx = Index.new(INDEX_BASE_FILE)
		idx_string = @plugin.filtering(idx.to_html)
		DataStore.drop2file(idx_string, INDEX_FILE)

		@plugin.exec
	end


	class Plugin
		def initialize(year, month, day, day_suffix, table)
			@year = year
			@month = month
			@day = day
			@day_suffix = day_suffix
			@table = table
		end

		def filtering(str)
			r = str.gsub(/<%= @plugin\(['"](.*?)['"]\) %>/) {
				plug_in_name = $1
				if File.exist?("#{PLUGIN_DIRECTORY}rtds_#{plug_in_name}.rb")
					load("#{PLUGIN_DIRECTORY}rtds_#{plug_in_name}.rb")
					%Q(<%= rtds_#{plug_in_name} %>)
				else
					%Q(<!-- plug-in "#{plug_in_name}" load failed. -->)
				end
			}

			erb = ERB.new(r)
			erb.result(binding)
		end

		def exec
			PLUGIN_EXEC.each {|plug_in_name|
				plug_in_method = "exec_" + plug_in_name.downcase
				load("#{PLUGIN_DIRECTORY}#{plug_in_method}.rb")
				eval(plug_in_method)
			}
		end
	end


	class DataStore
		def DataStore.read(date)
			r = ''
			Dir.glob("#{LOG_DIRECTORY}#{date}*#{LOG_SUFFIX}").sort.each {|fn|
				r << IO.readlines(fn).join
			}
			r
		end

		def DataStore.exist?(date)
			if Dir.glob("#{LOG_DIRECTORY}#{date}*#{LOG_SUFFIX}") == []
				return(false)
			else
				return(true)
			end
		end

		def DataStore.write(date, stream)
			fh = File.open(LOG_DIRECTORY + date + LOG_SUFFIX, 'w')
			fh.flock(File::LOCK_EX)
			fh.write stream
			fh.flock(File::LOCK_UN)
			fh.close
		end

		def DataStore.drop2file(str, file_path)
			fh = File.open(file_path, 'w')
			fh.flock(File::LOCK_EX)
			fh.write(str)
			fh.flock(File::LOCK_UN)
			fh.close
		end

		def DataStore.drop2log(year, month, day, str)
			n = '01'
			while File.exist?(LOG_DIRECTORY + year + month + day + n + LOG_SUFFIX)
				n.succ!
			end

			fh = File.open(LOG_DIRECTORY + year + month + day + n + LOG_SUFFIX, 'w')
			fh.flock(File::LOCK_EX)
			fh.write(str)
			fh.flock(File::LOCK_UN)
			fh.close
		end

		def DataStore.list
			Dir.glob("#{LOG_DIRECTORY}*#{LOG_SUFFIX}")
		end

		def DataStore.file_list
			file_list = []
			Dir.glob("#{BASE_DIRECTORY}*").each {|fp| file_list << fp if fp =~ /#{BASE_DIRECTORY}\d{6}[ab]\./ }
			file_list
		end
	end


	class BlueCarpet  # markdown format diary file converter.
		def initialize(date, content)
			@year = date[0, 4]
			@month = date[4, 2]
			@day = date[6, 2]
			if @day.to_i < 16
				@day_suffix = 'a'
			else
				@day_suffix = 'b'
			end
			@str = content
		end

		def to_html
			id =  @year + @month + @day
			x = titlize(translate(filtering(@str)), @year, @month, @day, @day_suffix)
			make_footnote(x, id)
		end

		def make_footnote(str, date_id)
			# Ex. date_id = '20040321'
			fn_id = '00'
			footnote = ''

			str = str.gsub(/\(\*(.*?)\)/) {
				fn_id.succ!
				id = date_id + '_' + fn_id
				fn_string = $1
				fn_string.gsub!(/"/, '&quot;')

				footnote << %Q( <li><p class="footnote"><a id="fn#{id}_02")
				footnote << %Q( name="fn#{id}_02")
				footnote << %Q( href="#fn#{id}_01">)
				footnote << %Q( #{fn_id.to_i.to_s}</a> #{fn_string}</p></li>\n)
				# id.to_i.to_s for '02' => '2'

				r = ''
				r << %Q(<sup><a id="fn#{id}_01")
				r << %Q( name="fn#{id}_01")
				r << %Q( href="#fn#{id}_02")
				r << %Q( title="#{fn_string}">)
				r << %Q(*#{fn_id.to_i.to_s}</a></sup>)
				# id.to_i.to_s for '02' => '2'
			}

			if footnote != ""
				x = %Q(<div class="footnote"><ol class="fn">\n#{footnote}</ol></div><!-- footnote end -->\n)
				str = str.sub(/<\/div><!-- #{BodyClass} end -->/, %Q(#{x}\n</div><!-- #{BodyClass} end -->))
			end

			str
		end

		def get_category(line)
			if line =~ /^#\s*(..)-.*/
				word = $1
				if CATEGORIES.include?(word)
					return(' [' + CATEGORIES[word] + ']')
				else
					return('')
				end
			else
				return('')
			end
		end

		def filtering(str)
			r = str.dup
			Dir.glob("#{FILTER_DIRECTORY}*.rb").each {|filter_name|
				load(filter_name.chomp)
				filter_method = File.basename(filter_name, '.rb')
				r = eval("#{filter_method}(r)")
			}
			r
		end

		def titlize(str, year, month, day, day_suffix)
			date_id = year + month + day
			fp = year + month + day_suffix + SUFFIX
			article_id = '00'
			r = ''

			str.each {|line|
				#if line =~ /^#/
				if line =~ /^<h1>/
					line.gsub!(/^<h1>/, '#')
					line.gsub!(/<\/h1>$/, '')
					article_id.succ!
					id = date_id + '_' + article_id
					category = get_category(line)

					line = line.sub(/^#..-/, '#') if category != ''
					line = translate(line.sub(/^#\s*/, '###'))  # h1 mark drop h3, and Markdown.

					section_anchor = %Q(<span class="#{SectionAnchorClass}">#{SectionAnchor}</span>)
					href_ids = %Q(<a id="d#{id}" name="d#{id}" href="#{fp}#d#{id}">)
					href_ide = %Q(</a>)

					line = line.sub(/^<h3>/, %Q(<h3 class="#{H3CLASS}">#{href_ids}#{section_anchor}#{href_ide} ))
					line = line.sub(/<\/h3>/, %Q(#{category}</h3>\n<div class="#{ArticleBodyClass}">))
				end
				r << line
			}
			date_line = Time.local(year, month, day).strftime("%Y / %m / %d ( %a )")
			date_line = %Q(<h2><span class="#{H2CLASS}"><a id="d#{date_id}" name="d#{date_id}" href="#{fp}#d#{date_id}">#{date_line}</a></span></h2>)

			section_start = %Q(<div class="#{SectionClass}">\n)
			section_end = %Q(</div><!-- #{ArticleBodyClass} end -->\n</div><!-- #{SectionClass} end -->\n)

			r = %Q(#{date_line}#{r}\n#{section_end}\n</div><!-- #{BodyClass} end -->)
			r.gsub!(/\n<h3/, %Q(#{section_end}\n#{section_start}<h3))  #FIXME! ugry...
			r.gsub!(/<\/h2><h3/, %Q(</h2>\n\n<div class="#{BodyClass}">\n\n#{section_start}<h3))
			r.gsub!(/<\/p>\n+<p>/, "</p>\n<p>")
			%Q(<div class="#{DayClass}">\n#{r}\n</div><!-- #{DayClass} end -->\n\n\n)
		end

		def translate(str)
			code = $KCODE
			$KCODE = 'NONE'
			r = BlueCloth.new(str).to_html
			$KCODE = code
			r
		end
	end
	

	class DiaryBase
		def initialize(base_file_path)
			@base_file_path = base_file_path
			@contents = ''
		end

		def to_html
			generate(@base_file_path, @contents)
		end


		private

		def generate(base_file_path, stream = '')
			base = IO.readlines(base_file_path).join

			if stream != ''
				return(base.gsub('<!-- TITLE -->', "<!-- #{TITLE} -->\n\n#{stream}"))
			else
				return(base)
			end
		end
	end


	class Diary < DiaryBase
		def initialize(base_file_path, year = '', month = '', day = '')
			super(base_file_path)

			if year == ''  # update from newest date log.
				n = '01'
				list = DataStore.list.sort
					list[list.size - 1].sub!(/#{LOG_DIRECTORY}(\d{4})(\d{2})(\d{2})\d{2}#{LOG_SUFFIX}/) {
						@year = $1
						@month = $2
						n = '16' if $3.to_i > 15
				}
			else
				@year = year; @month = month; @day = day
				if @day.to_i > 15
					n = '16'
				else
					n = '01'
				end
			end

			@diary_days = []
			15.times { @diary_days << n.dup; n.succ! }
			@diary_days << '31' if n == '30'

			@contents = ''
			@diary_days.each {|day|
				date = @year + @month + day
				@contents << BlueCarpet.new(date, DataStore.read(date)).to_html if DataStore.exist?(date)
			}
		end
	end

	class Index < DiaryBase
		def initialize(base_file_path)
			super
			@diary_days = []
			@contents = ''

			list = DataStore.list.sort
			llist = []
			list.each {|fn| fn.sub!(/#{LOG_DIRECTORY}(\d{8})\d{2}#{LOG_SUFFIX}/) { llist << $1 } }
			llist.uniq!
			INDEX_INCLUDES.times {|i| @diary_days << llist[llist.size - (1 + i)] }
			@diary_days.compact!

			@diary_days.uniq.each {|date|
				date.sub!(/#{LOG_DIRECTORY}(\d{8})\d{2}#{LOG_SUFFIX}/) { date = $1 }
				@contents << BlueCarpet.new(date, DataStore.read(date)).to_html
			}
		end
	end

	class DiaryIndex < DiaryBase ; end

	class DiaryTable
		def initialize
			@fl = []
			DataStore.file_list.each {|fn| @fl << fn.sub(/#{BASE_DIRECTORY}/, '') }
			@fl.sort!
		end

		def get(type, label = 'Go')
			@table = ''
			case type
			when 'wide'
				s = @fl[0].sub(/\..*/, '')
				e = @fl[@fl.size - 1].sub(/\..*/, '')

				s_year = ''
				e_year = ''

				s.sub(/^(\d{4})/) { s_year = $1 }
				e.sub(/^(\d{4})/) { e_year = $1 }

				nolinks = []
				if s_year == e_year
					t_month = '01'
					while t_month != '13'
						nolinks << "#{s_year}/#{t_month}a"
						nolinks << "#{s_year}/#{t_month}b"
						t_month.succ!
					end
				else
					ts_year = s_year.dup
					te_year = e_year.succ
					while ts_year != te_year
						t_month = '01'
						while t_month != '13'
							nolinks << "#{ts_year}/#{t_month}a"
							nolinks << "#{ts_year}/#{t_month}b"
							t_month.succ!
						end
						ts_year.succ!
					end
				end

				tmp = []
				nolinks.each {|dn|
					added = false
					@fl.each {|fn|
						if fn =~ /#{dn.delete('/')}/
							case dn.sub(/^\d{4}\//, '')
							when '03b', '06b', '09b', '12b'
								tmp << %Q(<a href="#{fn}">#{dn}</a><br />\n)
							else
								tmp << %Q(<a href="#{fn}">#{dn}</a>)
							end
							added = true
						end
					}
					if added == false
						case dn.sub(/^\d{4}\//, '')
						when '03b', '06b', '09b', '12b'
							tmp << %Q(#{dn}<br />\n)
						else
							tmp << %Q(#{dn})
						end
					end
				}

				@table = tmp.join(' | ') .gsub(/^ \| /, '')
			when 'pulldown'
				@table = %Q(<form action="#{File.basename($0)}">\n <p>\n  <select name="uri">\n)
				@fl.sort.reverse_each {|fn|
					date = ''
					fn.sub(/(\d{4})(\d{2}.)\..*/) { date = $1 + '/' + $2 }
					@table << %Q(   <option value="#{fn}">#{date}</option>\n)
				}
				@table << %Q(  </select>\n  <input type="submit" value="#{label}" />\n)
				@table << %Q(  <input type="hidden" name="mode" value="redirect" />\n </p>\n</form>\n)
			end

			@table
		end
	end
end

RTDS.new
